﻿#region Revision Info

// This file is part of Singular - A community driven Honorbuddy CC
// $Author: apoc $
// $Date: 2011-03-27 02:02:25 +0200 (Sun, 27 Mar 2011) $
// $HeadURL: http://svn.apocdev.com/singular/trunk/Singular/Extensions.cs $
// $LastChangedBy: apoc $
// $LastChangedDate: 2011-03-27 02:02:25 +0200 (Sun, 27 Mar 2011) $
// $LastChangedRevision: 235 $
// $Revision: 235 $

#endregion

using System;
using System.Text;

using Styx.WoWInternals.WoWObjects;

namespace Singular
{
    internal static class Extensions
    {
        /// <summary>A ulong extension method that converts a WoW performance counter to a DateTime object. </summary>
        /// <remarks>Created 3/26/2011.</remarks>
        /// <param name="performanceCounter">The performanceCounter to act on.</param>
        /// <returns>.</returns>
        public static DateTime PerformanceCounterToDateTime(this ulong performanceCounter)
        {
            return DateTime.Now.Subtract(TimeSpan.FromMilliseconds(Environment.TickCount)).AddMilliseconds(performanceCounter);
        }

        public static bool Between(this double distance, double min, double max)
        {
            return distance >= min && distance <= max;
        }

        /// <summary>
        ///   A string extension method that turns a Camel-case string into a spaced string. (Example: SomeCamelString -> Some Camel String)
        /// </summary>
        /// <remarks>
        ///   Created 2/7/2011.
        /// </remarks>
        /// <param name = "str">The string to act on.</param>
        /// <returns>.</returns>
        public static string CamelToSpaced(this string str)
        {
            var sb = new StringBuilder();
            foreach (char c in str)
            {
                if (char.IsUpper(c))
                {
                    sb.Append(' ');
                }
                sb.Append(c);
            }
            return sb.ToString();
        }

        public static string SafeName(this WoWObject obj)
        {
            if (obj.IsMe)
            {
                return "Myself";
            }

            if (obj is WoWPlayer)
            {
                return "Player";
            }

            if (obj is WoWUnit && obj.ToUnit().IsPet)
            {
                return "a Pet";
            }

            return obj.Name;
        }

        public static bool IsPet(this WoWUnit unit)
        {
            return unit.SummonedByUnitGuid != 0 || unit.CharmedByUnitGuid != 0;
        }
    }
}